TheScheduler.namespace('TheScheduler.View.BulkCloneDialog');

TheScheduler.View.BulkCloneDialog = (function () {
    return function (_scheduledIssues, _onBulkClone) {

        var BulkCloneDialog = TheScheduler.namespace('TheScheduler.View.BulkCloneDialog');

        if (!(this instanceof BulkCloneDialog)) {
            return new BulkCloneDialog(_scheduledIssues, _onBulkClone);
        }

        var scheduledIssues = _scheduledIssues;
        var onBulkClone = _onBulkClone;
        var dialogId='#tsp-si-bulk-clone-dialog';
        var dialog = null;

        var getDialogId = function () {
            return dialogId;
        };

        var show = function () {
            dialog.show();
        };

        var hide = function () {
            dialog.hide();
            AJS.dialog2(dialogId).remove();
        };

        var projectIssueField = function () {
            AJS.$.ajax(AJS.contextPath() + '/rest/thescheduler/1.0/project/jira-or-project-admin', {
                dataType: 'json',
            }).done(function (projects) {
                var projSelect = jQuery('#projectPicker');
                AJS.$.each(projects, function (idx, project) {
                    if (scheduledIssues[0].projectKey === project.key) {
                        projSelect.append('<option value=" ' + project.key + '" selected>' + project.name + '</option>');
                    } else {
                        projSelect.append('<option value=" ' + project.key + '">' + project.name + '</option>');
                    }
                });
                projSelect.select();
            });
        };

        var initDialog = function () {
            var bulkClonePanelContent = TheScheduler.Templates.Dialogs.cloneBulkDialog();
            AJS.$('body').append(bulkClonePanelContent);
            AJS.$('#prefix-name-tf').val(AJS.$('#jiraClonePrefix').val());

            var licenseNotValid = AJS.$('#license-is-not-valid');
            var cloneProgress = AJS.$('#clone-progress');
            var nameNotUniqueErrorDiv = AJS.$('#name-not-unique-error-div');
            var generalErrorDiv = AJS.$('#general-error');
            var duplicatedNameSiDiv = AJS.$('#si-with-not-unique-name-div');
            var dialog = AJS.dialog2(dialogId);
            var thisDialogElement = AJS.$(dialogId);

            function showSINamesErrors(duplicatedNames) {
                nameNotUniqueErrorDiv.show();
                if (duplicatedNames.length !== 0) {
                    var invalidNames = '';
                    for(var i = 0; i < duplicatedNames.length; i++) {
                        invalidNames = invalidNames + '<br><span style="margin-left: 1em"> - ' + duplicatedNames[i] + '</span>';
                    }
                    duplicatedNameSiDiv.append('<span>' + AJS.I18n.getText('pl.com.tt.thescheduler.errors.clone-name-with-prefix') + '</span>');
                    duplicatedNameSiDiv.append(invalidNames);
                    duplicatedNameSiDiv.show();
                }
            };

            function showGeneralErrors(errors) {
                $.each(errors, function (key, message) {
                    generalErrorDiv.append('<p>' + message + '</p>');
                });
                generalErrorDiv.show();
            };

            licenseNotValid.hide();
            cloneProgress.hide();
            nameNotUniqueErrorDiv.hide();
            generalErrorDiv.hide();
            duplicatedNameSiDiv.hide();

            var requestData = { projectId: scheduledIssues[0].projectId };
            jQueryTS.ajax({
                type: 'GET',
                url: AJS.contextPath() + '/rest/thescheduler/1.0/users/permissions',
                dataType: 'json',
                data: requestData,
                async: true,
                success: function (response) {
                    if (!AJS.$('select#projectPicker').html() && (response.projectAdmin || response.admin)) {
                        thisDialogElement.find('#tsp-si-bulk-clone-form').append('<div class="field-group">' +
                            '<label>Project</label>' +
                            '<select id="projectPicker" name="projectPicker" data-container-class="project-ss" class="select project-field" autocomplete="off">' +
                            '</select>' +
                            '</div>');
                        projectIssueField();
                    }
                },
                fail: function () {
                }
            });

            cloneProgress.hide();
            licenseNotValid.hide();

            AJS.$('#tsp-si-bulk-clone-close-button').click(function (e) {
                e.preventDefault();
                hide();
            });

            AJS.$('#tsp-si-bulk-clone-submit-button').click(function (e) {
                e.preventDefault();
                var prefixInput = AJS.$('#prefix-name-tf');
                var descriptionInput = AJS.$('#bulk-clone-description-tf');
                var projectPickerValue = scheduledIssues[0].projectKey;
                var projectPicker = AJS.$('#projectPicker option:selected');
                var addPrefixtoDesc = AJS.$('#tsp-si-bulk-clone-option-add-prefix-for-description').is(":checked");
                if (typeof projectPicker.val() != 'undefined') {
                    projectPickerValue = projectPicker.val().trim();
                }

                cloneProgress.show();
                licenseNotValid.hide();
                generalErrorDiv.empty();
                duplicatedNameSiDiv.empty();
                nameNotUniqueErrorDiv.hide();

                var tempDescription = "";
                var tempPrefixForDescription = "";

                if (addPrefixtoDesc) {
                  tempPrefixForDescription = descriptionInput.val();
                }
                else {
                  tempDescription = descriptionInput.val();
                }

                var data = {
                    scheduledIssuesIds: scheduledIssues.map(function(scheduledIssue) {
                        return scheduledIssue.id;
                    }),
                    prefix: prefixInput.val(),
                    description: tempDescription,
                    prefixForDescription: tempPrefixForDescription,
                    cloneProjectKey: projectPickerValue,
                    isContainsPrefixForDesc: addPrefixtoDesc,
                };

                AJS.$.ajax({
                    type: 'POST',
                    url: AJS.contextPath() + '/rest/thescheduler/1.0/scheduled-issue/clone/bulk',
                    datatype: 'json',
                    contentType: 'application/json',
                    data: JSON.stringify(data),
                    async: false,
                    success: function (response) {
                        hide();
                        onBulkClone(response);

                    },
                    error: function (jqXHR) {
                        if (jqXHR.status === 412) {
                            showSINamesErrors(JSON.parse(jqXHR.responseText));
                        } else {
                            showGeneralErrors(JSON.parse(jqXHR.responseText).errorMessages);
                        }
                        cloneProgress.hide();
                    }
                });
            });

            dialog.show();
            return dialog;
        };
        dialog = initDialog();
        return {
            'getDialogId': getDialogId,
            'show': show,
            'hide': hide
        };
    };
})();
