TheScheduler.namespace("TheScheduler.View.ConfigureColumnsInlineDialog");

TheScheduler.View.ConfigureColumnsInlineDialog = (function () {

    /* DECLARING DEPENDENCIES */

    /* STATIC PRIVATE MEMBERS */
    var nextInlineDialogId = 1;

    /* STATIC PRIVATE METHODS */

    /* STATIC INITIALIZATION */

    /* RETURNING AND OVERRIDING CONSTRUCTOR */
    return function (_scheduledIssueTable, _projectId) {

        var Notification = TheScheduler.namespace("TheScheduler.Utils.Notification");

        var ConfigureColumnsInlineDialog = TheScheduler.namespace("TheScheduler.View.ConfigureColumnsInlineDialog");

        /* IN CASE SOMEONE FORGOT USING *NEW* KEYWORD */
        if (!(this instanceof ConfigureColumnsInlineDialog)) {
            return new ConfigureColumnsInlineDialog(_scheduledIssueTable);
        }

        /* PRIVATE MEMBERS */
        var scheduledIssueTable = _scheduledIssueTable;
        var projectId = _projectId;
        var inlineDialogId = nextInlineDialogId++;
        var inlineDialog = null;

        /* PRIVATE METHODS */
        var getInlineDialogId = function () {
            return inlineDialogId;
        };

        var initInlineDialog = function () {

            function onCheckboxSelected(event) {

                var checkbox = jQueryTS(this);

                var columnId = checkbox.attr("id");
                var visible = checkbox.is(":checked");
                scheduledIssueTable.setColumnVisible(columnId, visible);


            }

            function configurationResett(event) {
                var visibleTab = [];
                if (projectId) {
                    var ajaxRequest = jQueryTS.ajax({
                        "type": "GET",
                        "url": AJS.contextPath() + "/rest/thescheduler/1.0/project/" + projectId + "/default-columns",
                        "contentType": "application/json; charset=UTF-8",
                        "async": false,
                    });
                    ajaxRequest.done(function(data) {
                        visibleTab = scheduledIssueTable.getColumnVisibilityTableAfterRefreshClick(data.columns);
                        Notification.showNotification(
                          'success',
                          AJS.I18n.getText('pl.com.tt.thescheduler.tabpanel.restore-columns'),
                          AJS.I18n.getText('pl.com.tt.thescheduler.tabpanel.restore-columns.desc'),
                          '#scheduled-issues-table-messages'
                        );
                    });
                } else {
                    visibleTab = [true, true, true, false, false, false, true, false, false, true, true, false, false, false, false, false, false, false, false, false, true, false, true];
                }
                jQueryTS.each(visibleTab, function (i, val) {
                    var checkbox = inlineDialog.find(".checkbox#" + i);
                    if (checkbox.is(":checked") !== val) {
                        scheduledIssueTable.setColumnVisible(i, val);
                        checkbox.prop("checked", val);
                    }
                    ;
                });
            }

            function saveColumnsPropertiesForUser() {
                var columns = scheduledIssueTable.getVisibleColumnsNames();
                var ajaxRequest = jQueryTS.ajax({
                    type: 'POST',
                    url: AJS.contextPath() + '/rest/thescheduler/1.0/users/properties/columns?projectId=' + projectId,
                    contentType: 'application/json',
                    dataType: 'json',
                    data: JSON.stringify({
                        "columns": columns
                    }),
                    async: true
                });
                ajaxRequest.done(function(data) {
                    Notification.showNotification(
                      'success',
                      AJS.I18n.getText('pl.com.tt.thescheduler.tabpanel.save-columns'),
                      AJS.I18n.getText('pl.com.tt.thescheduler.tabpanel.save-columns.desc'),
                      '#scheduled-issues-table-messages'
                    );
                });
            }


            var inlineDialog = AJS.InlineDialog("#configure-columns-trigger", "configure-columns-inline-dialog-" + inlineDialogId,
                function (content, trigger, showPopup) {

                    var renderedResult = TheScheduler.Templates.ConfigureColumnsInlineDialog.configureColumnsInlineDialog({});
                    content.html(renderedResult);

                    var visibleColumnIds = scheduledIssueTable.getVisibleColumnIds();

                    jQueryTS("input", content).each(function () {
                        var checkbox = jQueryTS(this);
                        var checkboxId = parseInt(checkbox.attr("id"));

                        if (jQueryTS.inArray(checkboxId, visibleColumnIds) >= 0) {
                            checkbox.prop("checked", true);
                        } else {
                            checkbox.prop("checked", false);
                        }
                    });

                    jQueryTS("input", content).off("click", onCheckboxSelected);
                    jQueryTS("input", content).on("click", onCheckboxSelected);
                    jQueryTS("button#refresh-columns-button").on("click", configurationResett);
                    jQueryTS("button#save-columns-button").on("click", saveColumnsPropertiesForUser);

                    jQueryTS("#configure-columns-dialog-content", content).off("click", function (evt) {
                        evt.stopPropagation();
                    });
                    jQueryTS("#configure-columns-dialog-content", content).on("click", function (evt) {
                        evt.stopPropagation();
                    });

                    //jQueryTS("div.checkbox", content).click(onCheckboxSelected);

                    showPopup();
                    return false;
                },
                {
                    "cacheContent": false//,
                    /*"useLiveEvents": true,*/
                    /*"noBind": true*/
                }
            );

            nextInlineDialogId++;
            return inlineDialog;
        };

        var show = function () {
            inlineDialog.show();
        };

        var hide = function () {
            inlineDialog.hide();
        };

        /* INSTANCE INITIALIZATION */
        inlineDialog = initInlineDialog();

        /* RETURNING ONLY DESIRABLE MEMBERS AND METHODS */
        return {
            "getInlineDialogId": getInlineDialogId,
            "show": show,
            "hide": hide
        };
    };
})();
