TheScheduler.namespace("TheScheduler.View.ScheduledIssueTable");

TheScheduler.View.ScheduledIssueTable = (function () {

    /* DECLARING DEPENDENCIES */
    var ScheduledIssue = TheScheduler.namespace("TheScheduler.Model.ScheduledIssue");

    /* STATIC PRIVATE MEMBERS */
    var expiresInYear = 365;
    var hiddenScheduledIssuesMessageClosedCookieName = 'tsp-hidden-scheduled-issues-message-closed';
    var pagingStartOffset = 1;
    var CallDataType = {
        FILTER: 'filter',
        DISPLAY: 'display',
        TYPE: 'type',
        SORT: 'sort'
    };

    /* STATIC PRIVATE METHODS */

    /* STATIC INITIALIZATION */

    /* RETURNING AND OVERRIDING CONSTRUCTOR */
    return function (_scheduledIssueTableId) {

        var Notification = TheScheduler.namespace("TheScheduler.Utils.Notification");

        var ScheduledIssueTable = TheScheduler.namespace("TheScheduler.View.ScheduledIssueTable");

        /* IN CASE SOMEONE FORGOT USING *NEW* KEYWORD */
        if (!(this instanceof ScheduledIssueTable)) {
            return new ScheduledIssueTable(_scheduledIssueTableId);
        }

        /* PRIVATE MEMBERS */
        var scheduledIssueTableId = _scheduledIssueTableId;
        var scheduledIssueTable = null;
        var table = null;
        var prioritiesForFilter = [];
        var issueTypesForFilter = [];
        var projectsForFilter = [];
        var statesForFilter = [];
        var defaultLimit = 200;
        var totalIssuesAmount = 0;
        var scheduledIssueUrlPath = AJS.contextPath() + "/rest/thescheduler/1.0/scheduled-issue";
        var projectId = jQueryTS("input#projectId").val();
        var projectSIUrl = AJS.contextPath() + "/rest/thescheduler/1.0/scheduled-issue/project/" + projectId;
        var failAttemptsLimit = 5;

        /* PRIVATE METHODS */
        var getScheduledIssueTableId = function () {
            return scheduledIssueTableId;
        };

        var getDataTable = function () {
            return scheduledIssueTable;
        };

        function selectAll() {
            var rowNodes = jQueryTS('tr', scheduledIssueTable);
            rowNodes.each(function () {
                selectRow(this);
            });

            checkSelectAllCheckbox();
        }

        function checkSelectAllCheckbox() {
            jQueryTS('input.select-all').prop('checked', true);
        }

        function selectRow(rowNode) {
            rowNode = jQueryTS(rowNode);

            var scheduledIssueId = rowNode.find('input.select-one').val();
            var checkboxElement = jQueryTS('input.select-one[value=' + scheduledIssueId + ']');

            rowNode.addClass('selected');
            checkboxElement.prop('checked', true);

            if (getSelectedRows().length === getVisibleRowsCount()) {
                checkSelectAllCheckbox();
            }
            updateSelectedRowsCounters();
        }

        function getVisibleRowsCount() {
            var pageInfo = scheduledIssueTable.api().page.info();
            return pageInfo.end - pageInfo.start;
        }

        function unselectAll() {
            var rowNodes = jQueryTS('tr', scheduledIssueTable);
            rowNodes.each(function () {
                unselectRow(this);
            });

            uncheckSelectAllCheckbox();
        }

        function uncheckSelectAllCheckbox() {
            jQueryTS('input.select-all').prop('checked', false);
        }

        function unselectRow(rowNode) {
            rowNode = jQueryTS(rowNode);

            var scheduledIssueId = rowNode.find('input.select-one').val();
            var checkboxElement = jQueryTS('input.select-one[value=' + scheduledIssueId + ']');

            rowNode.removeClass('selected');
            checkboxElement.prop('checked', false);

            uncheckSelectAllCheckbox();
            updateSelectedRowsCounters();
        }

        var getSelectedRows = function () {

            var trNodes = jQueryTS("tr", scheduledIssueTable);//scheduledIssueTable.fnGetNodes();
            var selectedCheckboxes = jQueryTS("input.select-one:checked", trNodes);
            var selectedRows = selectedCheckboxes.closest("tr");

            return selectedRows.toArray();
        };

        var getSelectedScheduledIssues = function () {

            var selectedScheduledIssues = [];

            var selectedRows = getSelectedRows();
            for (var i = 0; i < selectedRows.length; i++) {

              var scheduledIssue = scheduledIssueTable.fnGetData(selectedRows[i]);
              if (scheduledIssue) {
                selectedScheduledIssues.push(scheduledIssue);
              }
            }
            return selectedScheduledIssues;
        };

        var clearAndRedraw = function () {
            scheduledIssueTable.fnClearTable(true);
            return true;
        };

        var redraw = function () {
            scheduledIssueTable.fnDraw();
            return true;
        };

        var displayLengthChange = function (amount) {
            var tableSettings = scheduledIssueTable.fnSettings();
            tableSettings._iDisplayLength = amount;
            scheduledIssueTable.fnDraw();
            return true;
        };

        var addScheduledIssue = function (scheduledIssue, onAddScheduledIssueCallback) {
            if (scheduledIssue instanceof ScheduledIssue) {

                var scheduledIssueJSON = scheduledIssue.getScheduledIssueJSON();

                addPriorityToFilter(scheduledIssueJSON.priorityName);
                addIssueTypeToFilter(scheduledIssueJSON.issueTypeName);
                addProjectToFilter(scheduledIssueJSON.projectName);
                addStateToFilter(scheduledIssueJSON.state);

                var scheduledIssueIndexes = scheduledIssueTable.fnAddData(scheduledIssueJSON, false);
                var scheduledIssueIndex = scheduledIssueIndexes[0];
                var trRow = scheduledIssueTable.fnGetNodes(scheduledIssueIndex);

                if (onAddScheduledIssueCallback instanceof Function) {
                    onAddScheduledIssueCallback(scheduledIssueIndex, trRow);
                }

                if (scheduledIssueJSON.triggerPerformanceIssues == true && scheduledIssueJSON.disabled != true) {
                    performanceWarningMessageVisibility(true);
                }

                updateVisibleRowsCounter();

                return true;
            }
            return false;
        };

        var deleteScheduledIssue = function (scheduledIssue, onDeleteScheduledIssueCallback) {

            var scheduledIssueId = null;

            if (scheduledIssue instanceof ScheduledIssue) {
                scheduledIssueId = scheduledIssue.getId();
            } else if (typeof scheduledIssue === "number") {
                scheduledIssueId = scheduledIssue;
            }

            if (scheduledIssueId !== null) {
                var scheduledIssues = scheduledIssueTable.fnGetData();
                for (var i = 0; i < scheduledIssues.length; i++) {
                    if (scheduledIssues[i].id === scheduledIssueId) {

                        if (onDeleteScheduledIssueCallback instanceof Function) {

                            var scheduledIssueIndex = i;
                            var trRow = scheduledIssueTable.fnGetNodes(scheduledIssueIndex);

                            onDeleteScheduledIssueCallback(scheduledIssueIndex, trRow);
                        }

                        scheduledIssueTable.fnDeleteRow(i, null, false);
                        updateVisibleRowsCounter();
                        return true;
                    }
                }
            }

            return false;
        };

        var getScheduledIssues = function () {
            return scheduledIssueTable.fnGetData();
        };

        var getVisibleColumnIds = function () {

            var visibleColumnIds = [];

            var tableSettings = scheduledIssueTable.fnSettings();
            var tableColumns = tableSettings.aoColumns;
            for (var i = 0; i < tableColumns.length; i++) {
                var isColumnVisible = tableColumns[i].bVisible;
                if (isColumnVisible) {
                    visibleColumnIds.push(i);
                }
            }

            return visibleColumnIds;
        };

        var getVisibleColumnsNames = function() {

            var visibleColumnNames = [];

            var tableSettings = scheduledIssueTable.fnSettings();
            var tableColumns = tableSettings.aoColumns;
            for (var i = 0; i < tableColumns.length; i++) {
                var isColumnVisible = tableColumns[i].bVisible;
                if (isColumnVisible) {
                    visibleColumnNames.push(tableColumns[i].sName);
                }
            }

            return visibleColumnNames;
        }

        var getColumnVisibilityTableAfterRefreshClick = function(columnNames) {

            var visibleColumnTable = [];

            var tableSettings = scheduledIssueTable.fnSettings();
            var tableColumns = tableSettings.aoColumns;
            for (var i = 0; i < tableColumns.length; i++) {
                if (columnNames.includes(tableColumns[i].sName)) {
                    visibleColumnTable.push(true);
                } else {
                    visibleColumnTable.push(false);
                }
            }

            return visibleColumnTable;
        }

        var setColumnVisible = function (columnId, visible) {
            var filterElement = jQueryTS(scheduledIssueTableId + ' thead th input[data-column-index=' + columnId + ']');
            if (filterElement.length) {
                filterElement.val('');
                clearColumnFilter(columnId);
            }

            scheduledIssueTable.fnSetColumnVis(columnId, visible);
            scheduledIssueTable.fnAdjustColumnSizing();
            scheduledIssueTable.width("100%");

            setTimeout(redrawSelectFilters);

        };

        var setColumnVisibleWithoutRedraw = function (columnId, visible) {
            jQueryTS(scheduledIssueTableId).DataTable().column(columnId).visible(visible, false);
        };

        var setColumnVisibleByNameWithoutRedraw = function (columnNames) {
            var tableSettings = scheduledIssueTable.fnSettings();
            var tableColumns = tableSettings.aoColumns;
            for (var i = 0; i < tableColumns.length; i++) {
                if (columnNames.includes(tableColumns[i].sName)) {
                    setColumnVisibleWithoutRedraw(i, true);
                } else {
                    setColumnVisibleWithoutRedraw(i, false);
                }
            }
        };

        function redrawSelectFilters() {
            AJS.$('#tsp-schedules-issues-list-filter-priorities').empty();
            AJS.$('#tsp-schedules-issues-list-filter-issue-types').empty();
            AJS.$('#tsp-schedules-issues-list-filter-projects').empty();

            for (var i = 0; i < prioritiesForFilter.length; i++) {
                addPriorityFilterOption(prioritiesForFilter[i]);
            }
            for (var j = 0; j < issueTypesForFilter.length; j++) {
                addIssueTypeFilterOption(issueTypesForFilter[j]);
            }
            for (var k = 0; k < projectsForFilter.length; k++) {
                addProjectFilterOption(projectsForFilter[k]);
            }
        }

        jQueryTS("#scheduled-issues-table-blanket").show();

        var fetchAllScheduledIssues = function (requestUrl, limit, offset, attempt, refreshCompletedCallback, batchIterator) {
            var ajaxRequest = jQueryTS.ajax({
                "type": "GET",
                "url": requestUrl + "?limit=" + limit + "&offset=" + offset + "&min=true",
                "dataType": "json",
                "async": true
            });

            ajaxRequest.done(function (response) {
                var scheduledIssues = response.data;

                if (scheduledIssues.length > 0) {
                    scheduledIssueTable.api().rows.add(scheduledIssues);
                }

                if (response.total > offset + limit) {
                    fetchAllScheduledIssues(requestUrl, defaultLimit, offset + limit, 0, refreshCompletedCallback)
                } else {
                    if (typeof refreshCompletedCallback === "function") {
                        refreshCompletedCallback();
                    }
                }
            });

            ajaxRequest.fail(function (data) {
                console.error(data);
                if (attempt >= failAttemptsLimit) {
                    console.error("Fetching Scheduled Issues failed. Aborting")
                } else {
                    setTimeout(function () {
                        fetchAllScheduledIssues(requestUrl, defaultLimit, offset, attempt + 1, refreshCompletedCallback)
                    }, attempt * 500)
                }
            });

            ajaxRequest.always(function () {
                scheduledIssueTable.fnAdjustColumnSizing();
            })
        };

        var fetchFirstScheduledIssuesBatch = function (requestUrl, limit, offset, attempt) {
            var ajaxRequest = jQueryTS.ajax({
                "type": "GET",
                "url": requestUrl + "?limit=" + limit + "&offset=" + offset + "&min=true",
                "dataType": "json",
                "async": true
            });

            ajaxRequest.done(function (response) {
                var scheduledIssues = response.data;
                totalIssuesAmount = response.total;
                if (scheduledIssues.length > 0) {
                    scheduledIssueTable.api().rows.add(scheduledIssues).draw();
                    hideEmptyTableInfo();
                } else {
                    showEmptyTableInfo();
                }
                refreshPerformanceWarningMessage();
                updateFilters();
                jQueryTS("#scheduled-issues-table-blanket").hide();
            });
            ajaxRequest.fail(function (data) {
                console.error(data);
                if (attempt < failAttemptsLimit) {
                    setTimeout(function () {
                        console.error("Trying to fetch first sj batch ", attempt);
                        fetchFirstScheduledIssuesBatch(limit, offset, attempt + 1)
                    }, attempt * 500)
                } else {
                    console.error("Fetching first batch Scheduled Issues failed. Aborting")
                }
            });
            ajaxRequest.always(function () {
                scheduledIssueTable.fnAdjustColumnSizing();
                scheduledIssueTable.width("100%");
            })
        };

        var refreshSITable = function (requestUrl, refreshCompletedCallback) {
            jQueryTS("#scheduled-issues-table-blanket").show();
            scheduledIssueTable.api().clear();

            var firstBatchLimit = 50;
            fetchFirstScheduledIssuesBatch(requestUrl, firstBatchLimit, 0, 0);

            var onFetchingFinish = function () {
                if (typeof refreshCompletedCallback === "function") {
                    refreshCompletedCallback();
                }
                refreshPerformanceWarningMessage();
                updateFilters();
            };
          fetchAllScheduledIssues(requestUrl, defaultLimit, firstBatchLimit, 0, onFetchingFinish);
        };

      var refreshGlobalSITable = function (refreshCompletedCallback) {
        return refreshSITable(scheduledIssueUrlPath, refreshCompletedCallback);
      };

      function updateFilters() {
        var scheduledIssues = scheduledIssueTable.api()
          .rows()
          .data();
        for (var i = 0; i < scheduledIssues.length; i++) {
          addPriorityToFilter(scheduledIssues[i].priorityName);
          addIssueTypeToFilter(scheduledIssues[i].issueTypeName);
          addProjectToFilter(scheduledIssues[i].projectName);
          addStateToFilter(scheduledIssues[i].state);
        }
      }

      var refreshProjectSITable = function (refreshCompletedCallback) {
        return refreshSITable(projectSIUrl, refreshCompletedCallback);
      }

        function addPriorityToFilter(priorityName) {
            priorityName = priorityName || AJS.I18n.getText('pl.com.tt.thescheduler.variables.not-applicable');

            if (prioritiesForFilter.indexOf(priorityName) > -1) {
                return;
            }

            prioritiesForFilter.push(priorityName);
            addPriorityFilterOption(priorityName);
        }

        function addPriorityFilterOption(priorityName) {
            AJS.$('#tsp-schedules-issues-list-filter-priorities')
                .append(AJS.$('<option>').text(priorityName))
                .auiSelect2();
        }

        function addIssueTypeToFilter(issueTypeName) {
            issueTypeName = issueTypeName || AJS.I18n.getText('pl.com.tt.thescheduler.variables.not-applicable');

            if (issueTypesForFilter.indexOf(issueTypeName) > -1) {
                return;
            }

            issueTypesForFilter.push(issueTypeName);
            addIssueTypeFilterOption(issueTypeName);
        }

        function addIssueTypeFilterOption(issueTypeName) {
            AJS.$('#tsp-schedules-issues-list-filter-issue-types')
                .append(AJS.$('<option>').text(issueTypeName))
                .auiSelect2();
        }

        function addProjectToFilter(projectName) {
            if (projectsForFilter.indexOf(projectName) > -1) {
                return;
            }

            projectsForFilter.push(projectName);
            addProjectFilterOption(projectName);
        }

        function addProjectFilterOption(projectName) {
            AJS.$('#tsp-schedules-issues-list-filter-projects')
                .append(AJS.$('<option>').text(projectName))
                .auiSelect2();
        }

        function addStateToFilter(stateName) {
            if (statesForFilter.indexOf(stateName) > -1) {
                return;
            }

            statesForFilter.push(stateName);
            addStateFilterOption(stateName);
        }

        function addStateFilterOption(stateName) {
            AJS.$('#tsp-schedules-issues-list-filter-states')
                .append(AJS.$('<option>').text(stateName))
                .auiSelect2();
        }

        function filterColumn(columnIndex, text, searchUsingRegex) {
            searchUsingRegex = searchUsingRegex || false;
            var useSmartSearch = !searchUsingRegex;     // need to disable smart search if regex is used

            var selectedRows = getSelectedRows();
            for (var i = 0; i < selectedRows.length; i++) {
                unselectRow(selectedRows[i]);
            }

            var column = table.api().column(columnIndex);
            if (column.search() !== text) {
                column
                    .search(text, searchUsingRegex, useSmartSearch)
                    .draw();

                updateVisibleRowsCounter();
                selectVisibleRows(selectedRows);
            }
        }

        function selectVisibleRows(rows) {
            for (var i = 0; i < rows.length; i++) {
                selectRow(rows[i]);
            }
        }

        function clearColumnFilter(columnIndex) {
            filterColumn(columnIndex, '');
        }

        function updateVisibleRowsCounter() {
            var pageInfo = scheduledIssueTable.api().page.info();
            var totalRecordsCount = projectId ? pageInfo.recordsDisplay : totalIssuesAmount;
            var infoText = AJS.I18n.getText(
                'pl.com.tt.thescheduler.variables.visible-si-count',
                pageInfo.start + pagingStartOffset,
                pageInfo.end,
                totalRecordsCount
            );
            jQueryTS('#tsp-scheduled-issues-list-visible-si-count').text(infoText);
        }

        function updateSelectedRowsCounters() {
            var selectedRowsCount = getSelectedRows().length;
            jQueryTS('.selected-si-action-button-badge').text(selectedRowsCount);
        }

        function checkIfAnyTriggerIsSkipping(triggers) {
            var isAnySkipping = false;
            triggers.forEach(function(trigger) {
                if (trigger.skipScheduledExecutionAfterManual && !trigger.recentManualExecutionSj) {
                    isAnySkipping = true;
                }
            });
            return isAnySkipping;
        }

        var initDataTable = function () {

            jQueryTS.fn.dataTableExt.oStdClasses.sSortAsc = "ascending";
            jQueryTS.fn.dataTableExt.oStdClasses.sSortDesc = "descending";
            jQueryTS.fn.dataTableExt.oStdClasses.sSortable = "sortable";

            var manageScheduledJobs = jQueryTS("#manageAllScheduledJobs").val() == "true" ? true : false;
            var previousPage = null;

            /* FUNCTION DEFINITIONS RESPONSIBLE FOR RENDERING PARTICULAR PIECE OF SCHEDULED ISSUES TABLE - ONE FUNCTION FOR EACH TABLE COLUMN */
            function renderSelectColumn(data, type, row) {
                return TheScheduler.Templates.ScheduledIssueTable.selectColumn({
                    "scheduledIssue": row
                });
            }

            function renderNameColumn(data, type, row) {
                return TheScheduler.Templates.ScheduledIssueTable.nameColumn({
                    "contextPath": AJS.contextPath(),
                    "scheduledIssue": row,
                    "manageScheduledJobs": manageScheduledJobs
                });
            }

            function renderStateColumn(data, type, row) {
                if (type === CallDataType.FILTER) {
                    return data.state;
                }
                return TheScheduler.Templates.ScheduledIssueTable.stateColumn({
                    scheduledIssue: row,
                    contextPath: AJS.contextPath(),
                    manageScheduledJobs: manageScheduledJobs
                });
            }

            function renderDescriptionColumn(data, type, row) {
                return TheScheduler.Templates.ScheduledIssueTable.descriptionColumn({
                    "scheduledIssue": row
                });
            }

            function renderSummaryColumn(data, type, row) {
                return TheScheduler.Templates.ScheduledIssueTable.summaryColumn({
                    "scheduledIssue": row
                });
            }

            function renderTriggersColumn(data, type, row) {
                return TheScheduler.Templates.ScheduledIssueTable.triggersColumn({
                    "scheduledIssue": row
                });
            }

            function renderTriggersExpressionsColumn(data, type, row) {
                return TheScheduler.Templates.ScheduledIssueTable.triggersExpressionsColumn({
                    "scheduledIssue": row
                });
            }

            function renderActionColumn(data, type, row) {
                var textToShow = "";
                if (row.anyTriggerSkipping) {
                    textToShow += AJS.I18n.getText("pl.com.tt.thescheduler.scheduled-job.manual-execution-skipping-notification");
                }
                if (row.swappingAnyUserField) {
                    textToShow += AJS.I18n.getText("pl.com.tt.thescheduler.scheduled-job.manual-execution-swapping-fields-notification");
                }
                return TheScheduler.Templates.ScheduledIssueTable.actionsColumn({
                    "contextPath": AJS.contextPath(),
                    "scheduledIssue": row,
                    "manageScheduledJobs": manageScheduledJobs,
                    "textToShow": textToShow
                });
            }

            function renderProjectColumn(data, type, row) {
                return TheScheduler.Templates.ScheduledIssueTable.projectColumn({
                    "scheduledIssue": row
                });
            }

            function renderIssueTypeColumn(data, type, row) {
                return TheScheduler.Templates.ScheduledIssueTable.issueTypeColumn({
                    "contextPath": AJS.contextPath(),
                    "scheduledIssue": row
                });
            }

            function renderPriorityColumn(data, type, row) {
                return TheScheduler.Templates.ScheduledIssueTable.priorityColumn({
                    "scheduledIssue": row
                });
            }

            function renderReporterColumn(data, type, row) {
                return TheScheduler.Templates.ScheduledIssueTable.reporterColumn({
                    "contextPath": AJS.contextPath(),
                    "scheduledIssue": row
                });
            }

            function renderAssigneeColumn(data, type, row) {
                return TheScheduler.Templates.ScheduledIssueTable.assigneeColumn({
                    "contextPath": AJS.contextPath(),
                    "scheduledIssue": row,
                });
            }

            function renderCreatedByColumn(data, type, row) {
                return TheScheduler.Templates.ScheduledIssueTable.createdByColumn({
                    "contextPath": AJS.contextPath(),
                    "scheduledIssue": row
                });
            }

            function renderCreatedAtColumn(data, type, row) {
                return TheScheduler.Templates.ScheduledIssueTable.createdAtColumn({
                    "scheduledIssue": row
                });
            }

            function renderModifiedByColumn(data, type, row) {
                return TheScheduler.Templates.ScheduledIssueTable.modifiedByColumn({
                    "contextPath": AJS.contextPath(),
                    "scheduledIssue": row
                });
            }

            function renderModifiedAtColumn(data, type, row) {
                return TheScheduler.Templates.ScheduledIssueTable.modifiedAtColumn({
                    "scheduledIssue": row
                });
            }

            function renderPrevFireTimeColumn(data, type, row) {
                return TheScheduler.Templates.ScheduledIssueTable.prevFireTimeColumn({
                    "scheduledIssue": row
                });
            }

            function renderNextFireTimeColumn(data, type, row) {
                return TheScheduler.Templates.ScheduledIssueTable.nextFireTimeColumn({
                    "scheduledIssue": row
                });
            }

            function renderFireCountColumn(data, type, row) {
                return TheScheduler.Templates.ScheduledIssueTable.fireCountColumn({
                    "scheduledIssue": row
                });
            }

            function renderLinksColumn(data, type, row) {
                return TheScheduler.Templates.ScheduledIssueTable.linksColumn({
                    "scheduledIssue": row
                });
            }

            function renderSharedColumn(data, type, row) {
                return TheScheduler.Templates.ScheduledIssueTable.sharedColumn({
                    "scheduledIssue": row
                });
            }

            function renderDueDateColumn(data, type, row) {
                return TheScheduler.Templates.ScheduledIssueTable.dueDateColumn({
                    "scheduledIssue": row
                });
            }

            function onSelectAllClick() {
                var selectAllCheckbox = jQueryTS(this);
                var selectAllChecked = selectAllCheckbox.is(":checked");

                if (selectAllChecked) {
                    selectAll();
                } else {
                    unselectAll();
                }
            }

            jQueryTS(scheduledIssueTableId).on("click", "input.select-all", onSelectAllClick);

            function onSelectOneClick() {
                var checkboxElement = jQueryTS(this);
                var trElement = checkboxElement.closest("tr");

                if (checkboxElement.is(':checked')) {
                    selectRow(trElement);
                } else {
                    unselectRow(trElement);
                }
            }

            jQueryTS(scheduledIssueTableId).on("click", "input.select-one", onSelectOneClick);

            function onEnableScheduledIssueClick(eventObject) {

                eventObject.preventDefault();
                eventObject.stopPropagation();

                var enableLinkElement = jQueryTS(this);
                var scheduledIssueId = enableLinkElement.attr("href");

                function setSJDisableRequest() {
                    return jQueryTS.ajax({
                        "type": "POST",
                        "url": AJS.contextPath() + "/rest/thescheduler/1.0/scheduled-issue/" + scheduledIssueId,
                        "contentType": "application/json; charset=UTF-8",
                        "dataType": "json",
                        "async": false,
                        "data": JSON.stringify({
                            "id": scheduledIssueId,
                            "disabled": false
                        })
                    });
                }


                licenseCheckRequest().done(function (response) {
                    if (response.valid) {
                        setSJDisableRequest().done(function (data) {
                            var scheduledIssues = scheduledIssueTable.fnGetData();
                            for (var i = 0; i < scheduledIssues.length; i++) {
                                if (scheduledIssues[i].id == scheduledIssueId) {
                                    scheduledIssueTable.fnUpdate(data, i);
                                }
                            }
                            refreshPerformanceWarningMessage();
                        });
                    } else {
                        showLicenseError();
                    }
                });
                return false;
            }

            jQueryTS("body").off("click", "a.enable-link");
            jQueryTS("body").on("click", "a.enable-link", onEnableScheduledIssueClick);

            function onDisableScheduledIssueClick(eventObject) {

                eventObject.preventDefault();
                eventObject.stopPropagation();

                var disableLinkElement = jQueryTS(this);
                var scheduledIssueId = disableLinkElement.attr("href");

                licenseCheckRequest().done(function (response) {
                    if (response.valid) {
                        var ajaxRequest = jQueryTS.ajax({
                            "type": "POST",
                            "url": AJS.contextPath() + "/rest/thescheduler/1.0/scheduled-issue/" + scheduledIssueId,
                            "contentType": "application/json; charset=UTF-8",
                            "dataType": "json",
                            "async": false,
                            "data": JSON.stringify({
                                "id": scheduledIssueId,
                                "disabled": true
                            })
                        });
                        ajaxRequest.done(function (data) {
                            var scheduledIssues = scheduledIssueTable.fnGetData();
                            for (var i = 0; i < scheduledIssues.length; i++) {
                                if (scheduledIssues[i].id == scheduledIssueId) {
                                    scheduledIssueTable.fnUpdate(data, i);
                                }
                            }
                            refreshPerformanceWarningMessage();
                        });
                    } else {
                        showLicenseError();
                    }
                });
                return false;
            }


            function showLicenseError() {
                var ErrorDialog = TheScheduler.namespace('TheScheduler.View.ErrorDialog');
                var errorDialog = new ErrorDialog(
                    AJS.I18n.getText("pl.com.tt.thescheduler.errors.license"),
                    AJS.I18n.getText('pl.com.tt.thescheduler.errors.license.disabled-feature')
                );
                errorDialog.show();
            }


            jQueryTS("body").off("click", "a.disable-link");
            jQueryTS("body").on("click", "a.disable-link", onDisableScheduledIssueClick);

            function onFireScheduledIssueClick(eventObject) {
                eventObject.preventDefault();
                eventObject.stopPropagation();

                var fireLinkElement = jQueryTS(this);
                var scheduledIssueId = fireLinkElement.attr("href");
                var scheduledIssueName = fireLinkElement.attr("siname");
                var isAnySkipping = false;
                var isAnyFieldSwapping = false;
                var scheduledIssues = scheduledIssueTable.fnGetData();
                for (var i = 0; i < scheduledIssues.length; i++) {
                    if (scheduledIssues[i].id.toString() === scheduledIssueId) {
                        isAnyFieldSwapping = scheduledIssues[i].swappingAnyUserField;
                        isAnySkipping = scheduledIssues[i].anyTriggerSkipping;
                        break;
                    }
                }

                var ajaxRequest = jQueryTS.ajax({
                    "type": "GET",
                    "url": AJS.contextPath() + "/rest/thescheduler/1.0/validations/license",
                    "async": true,
                });
                ajaxRequest.done(function (response) {
                    if (response.valid) {
                        var ConfirmationDialog = TheScheduler.namespace("TheScheduler.View.FireConfirmationDialog");
                        var confirmationDialog = new ConfirmationDialog(scheduledIssueId, scheduledIssueName,
                          isAnySkipping, isAnyFieldSwapping, function () {
                            fireScheduledIssue(fireLinkElement);
                        });
                        confirmationDialog.show();
                    } else {
                        var ErrorDialog = TheScheduler.namespace("TheScheduler.View.ErrorDialog");
                        var errorDialog = new ErrorDialog(AJS.I18n.getText("pl.com.tt.thescheduler.errors.license"), AJS.I18n.getText("pl.com.tt.thescheduler.errors.license.disabled-feature"));
                        errorDialog.show();
                    }
                });
                return false;
            }

            function fireScheduledIssue(fireLinkElement) {
                var scheduledIssueId = fireLinkElement.attr("href");
                var scheduledIssueName = fireLinkElement.attr("siname");

                var fireLinkProgressElement = fireLinkElement.prev();
                fireLinkProgressElement.css("visibility", "visible");

                var trElement = fireLinkElement.closest("tr");
                var selectedScheduledIssuesBean = {
                    scheduledIssueIdsToExecute: Array.of(scheduledIssueId),
                };

                var ajaxRequest = jQueryTS.ajax({
                    type: 'POST',
                    url: AJS.contextPath() + '/rest/thescheduler/1.0/scheduled-issue/fire',
                    contentType: 'application/json',
                    data: JSON.stringify(selectedScheduledIssuesBean),
                    dataType: 'json',
                    async: true
                });

                ajaxRequest.done(function (data) {
                    var fireHistoryEntry = data.shift();
                    var rowData = table.fnGetData(trElement);
                    rowData.fireCount++;
                    table.fnUpdate(rowData, trElement);
                    AJS.$(trElement).find(".fire-link").tooltip({gravity: "e"});
                    AJS.$(trElement).find(".scheduler-tooltip-enabled").tooltip();
                    const issueLink = '<b><a href="' + AJS.contextPath() + '/browse/' + fireHistoryEntry.issueKey + '">' + fireHistoryEntry.issueKey + '</a></b>';
                    if (fireHistoryEntry.executionResult === "INCREASE_PRIORITY") {
                        Notification.showNotification(
                            'success',
                            AJS.I18n.getText("pl.com.tt.thescheduler.tabpanel.increased"),
                            AJS.I18n.getText("pl.com.tt.thescheduler.tabpanel.increased.desc"),
                            '#scheduled-issues-table-messages'
                        );
                    } else if (fireHistoryEntry.executionResult === "NONE") {
                        Notification.showNotification(
                            'info',
                            AJS.I18n.getText("pl.com.tt.thescheduler.tabpanel.nothing-done"),
                            AJS.I18n.getText("pl.com.tt.thescheduler.tabpanel.nothing-done.desc"),
                            '#scheduled-issues-table-messages'
                        );
                    } else if (fireHistoryEntry.executionResult === "CREATE_ISSUE" && fireHistoryEntry.errorMessage) {
                        const body = AJS.I18n.getText("pl.com.tt.thescheduler.tabpanel.fired.desc", issueLink) + " " + fireHistoryEntry.errorMessage;
                        Notification.showNotification(
                            'warning',
                            AJS.I18n.getText("pl.com.tt.thescheduler.actions.warn"),
                            body,
                            '#scheduled-issues-table-messages'
                        );
                    } else {
                        Notification.showNotification(
                            'success',
                            AJS.I18n.getText("pl.com.tt.thescheduler.tabpanel.fired"),
                            AJS.I18n.getText("pl.com.tt.thescheduler.tabpanel.fired.desc", issueLink),
                            '#scheduled-issues-table-messages'
                        );
                    }
                });

                ajaxRequest.fail(function (jqXHRObject) {
                    var data = jqXHRObject.responseJSON;
                    if (!(typeof data === 'undefined') && !(typeof data.errors === 'undefined') && !(typeof data.errors.license === 'undefined')) {
                        var ErrorDialog = TheScheduler.namespace("TheScheduler.View.ErrorDialog");
                        var errorDialog = new ErrorDialog(AJS.I18n.getText("pl.com.tt.thescheduler.errors.license"), AJS.I18n.getText("pl.com.tt.thescheduler.errors.license.disabled-feature"));
                        errorDialog.show();
                        return false;
                    }
                });

                ajaxRequest.always(function () {
                    fireLinkProgressElement.css("visibility", "hidden");
                });
            }

            jQueryTS("body").off("click", "a.fire-link");
            jQueryTS("body").on("click", "a.fire-link", onFireScheduledIssueClick);

            function onShowCalendarClick(eventObject) {

                eventObject.preventDefault();
                eventObject.stopPropagation();

                var fireLinkElement = jQueryTS(this);
                trElement = fireLinkElement.closest("tr");
                var rowData = table.fnGetData(trElement);

                var ShowCalendarDatesDialog = TheScheduler.namespace("TheScheduler.View.ShowCalendarDatesDialog");

                var showCalendarDatesDialog = new ShowCalendarDatesDialog(rowData.calendar.name,
                    rowData.calendar.calendarDates, rowData.calendar.calendarType);
                showCalendarDatesDialog.show();


            }

            jQueryTS("body").off("click", "span.calendar-link");
            jQueryTS("body").on("click", "span.calendar-link", onShowCalendarClick);

            function rowCreated(nRow, aData, iDataIndex) {
                AJS.$(nRow).find(".fire-link").tooltip({gravity: "e"});
                AJS.$(nRow).find(".scheduler-tooltip-enabled").tooltip();

                //APDC-2293 APDC-2307
                //AJS.$(nRow).click(function(eventObject) {
                //	var checkbox = AJS.$( this ).find(".select-one");
                //	checkbox.prop("checked", !checkbox.prop("checked"));
                //});
            }

            function drawCallback(oSettings) {
                if (oSettings.fnRecordsDisplay() == 0) {
                    AJS.$("#scheduled-issues-table_paginate").hide();
                } else {
                    AJS.$("#scheduled-issues-table_paginate").show();
                }
            }

            applySearchUsingVisibleColumns();
            buildFiltersRow();

            function applySearchUsingVisibleColumns() {
                // There is no current way in DataTables to disable searching dynamically,
                // so just check if the column is visible and exclude it from another search
                jQueryTS.fn.dataTable.ext.search.push(searchUsingVisibleColumnsOnly);
            }

            function searchUsingVisibleColumnsOnly(settings, data) {
                // Always return true if search is blank (save processing)
                if (settings.oPreviousSearch.sSearch === '') {
                    return true;
                }

                var searchString = jQueryTS.fn.DataTable.util.escapeRegex(settings.oPreviousSearch.sSearch);

                var visibleColumnsData = [];
                for (var i = 0; i < settings.aoColumns.length; i++) {
                    if (settings.aoColumns[i].bVisible && settings.aoColumns[i].bSearchable) {
                        visibleColumnsData.push(data[i]);
                    }
                }

                var regex = new RegExp('^(?=.*?' + searchString + ').*$', 'i');
                return regex.test(visibleColumnsData.join(' '));
            }

            function buildFiltersRow() {
                AJS.$(scheduledIssueTableId + ' thead th:not(.select-column) input').each(function () {
                    var inputElement = AJS.$(this);
                    var columnIndex = parseInt(inputElement.data('column-index'), 10);

                    inputElement.on('keyup change', onColumnFilter(columnIndex));
                });

                AJS.$(scheduledIssueTableId + ' thead th select').each(function () {
                    var selectElement = AJS.$(this);
                    var columnIndex = parseInt(selectElement.data('column-index'), 10);

                    selectElement
                        .on('change', function (event) {
                            var regex = '';
                            if (event.val.length > 0) {
                                if (selectElement.data('not-exact-match')) {
                                    regex = '^' + event.val.join('|^');
                                } else {
                                    regex = '^' + event.val.join('$|^') + '$';
                                }
                            }
                            var useRegex = true;

                            filterColumn(columnIndex, regex, useRegex);
                        });
                });
            }

            function onColumnFilter(columnIndex) {
                return function (event) {
                    filterColumn(columnIndex, event.target.value);
                }
            }

            function onPageChange() {
                var currentPage = table.api().page.info();
                if (previousPage.page !== currentPage.page) {
                    unselectAll();
                }

                previousPage = currentPage;
                updateVisibleRowsCounter();
            }

            function clearAllFilters() {
                table.api().columns().search('');
            }

            /* HERE WE ARE INITIALIZING DATA TABLE OBJECT FOR SCHEDULED ISSUES TABLE */
            table = jQueryTS(scheduledIssueTableId).dataTable({
                bSort: true,
                bSortClasses: false,
                bFilter: true,
                bInfo: true,
                bLengthChange: false,
                bPaginate: true,
                sPaginationType: "full_numbers",
                iDisplayLength: 10,
                bAutoWidth: true,
                sDom: "tpi",
                bDeferRender: true,
                bStateSave: true,
                fnCreatedRow: rowCreated,
                fnDrawCallback: drawCallback,
                oLanguage: {
                    sZeroRecords: AJS.I18n.getText("pl.com.tt.thescheduler.tabpanel.empty-filter"),
                    oPaginate: {
                        sFirst: AJS.I18n.getText("pl.com.tt.thescheduler.tabpanel.pagination.first"),
                        sLast: AJS.I18n.getText("pl.com.tt.thescheduler.tabpanel.pagination.last"),
                        sNext: AJS.I18n.getText("pl.com.tt.thescheduler.tabpanel.pagination.next"),
                        sPrevious: AJS.I18n.getText("pl.com.tt.thescheduler.tabpanel.pagination.previous")
                    }
                },
                /*"fnCookieCallback": fnCookieCallback,
                 "fnStateLoad": fnStateLoad,*/
                orderCellsTop: true,
                bDestroy: true,
                aoColumnDefs: [
                    {
                        bSearchable: false,
                        sClass: "select-column",
                        mData: null,
                        mRender: renderSelectColumn,
                        bSortable: false,
                        sWidth: "16px",
                        aTargets: [0],
                        sName: "selectColumn",
                        type: 'html'
                    },
                    {
                        sClass: "name-column",
                        mData: null,
                        mRender: renderNameColumn,
                        bSortable: true,
                        aTargets: [1],
                        sName: "nameColumn",
                        type: 'html'
                    },
                    {
                        sClass: "state-column",
                        mData: null,
                        mRender: renderStateColumn,
                        bSortable: true,
                        aTargets: [2],
                        sName: "stateColumn",
                        type: 'html'
                    },
                    {
                        sClass: "description-column",
                        mData: null,
                        mRender: renderDescriptionColumn,
                        bSortable: true,
                        bVisible: false,
                        aTargets: [3],
                        sName: "descriptionColumn",
                        type: 'html'
                    },
                    {
                        sClass: "summary-column",
                        mData: null,
                        mRender: renderSummaryColumn,
                        bSortable: true,
                        bVisible: false,
                        aTargets: [4],
                        sName: "summaryColumn",
                        type: 'html'
                    },
                    {
                        sClass: "project-column",
                        mData: null,
                        mRender: renderProjectColumn,
                        bSortable: true,
                        bVisible: false,
                        aTargets: [5],
                        sName: "projectColumn",
                        type: 'html'
                    },
                    {
                        sClass: "issue-type-column",
                        mData: null,
                        mRender: renderIssueTypeColumn,
                        bSortable: true,
                        aTargets: [6],
                        sName: "issueTypeColumn",
                        type: 'html'
                    },
                    {
                        sClass: "priority-column",
                        mData: null,
                        mRender: renderPriorityColumn,
                        bSortable: true,
                        bVisible: false,
                        aTargets: [7],
                        sName: "priorityColumn",
                        type: 'html'
                    },
                    {
                        sClass: "reporter-column",
                        mData: null,
                        mRender: renderReporterColumn,
                        bSortable: true,
                        bVisible: false,
                        aTargets: [8],
                        sName: "reporterColumn",
                        type: 'html'
                    },
                    {
                        sClass: "assignee-column",
                        mData: null,
                        mRender: renderAssigneeColumn,
                        bSortable: true,
                        aTargets: [9],
                        sName: "assigneeColumn",
                        type: 'html'
                    },
                    {
                        sClass: "created-by-column",
                        mData: null,
                        mRender: renderCreatedByColumn,
                        bSortable: true,
                        aTargets: [10],
                        sName: "createdByColumn",
                        type: 'html'
                    },
                    {
                        bSearchable: false,
                        sClass: "created-at-column",
                        mData: null,
                        mRender: renderCreatedAtColumn,
                        bSortable: true,
                        bVisible: false,
                        aTargets: [11],
                        sName: "createdAtColumn",
                        type: 'html'
                    },
                    {
                        sClass: "modified-by-column",
                        mData: null,
                        mRender: renderModifiedByColumn,
                        bSortable: true,
                        bVisible: false,
                        aTargets: [12],
                        sName: "modifiedByColumn",
                        type: 'html'
                    },
                    {
                        bSearchable: false,
                        sClass: "modified-at-column",
                        mData: null,
                        mRender: renderModifiedAtColumn,
                        bSortable: true,
                        bVisible: false,
                        aTargets: [13],
                        sName: "modifiedAtColumn",
                        type: 'html'
                    },
                    {
                        bSearchable: false,
                        sClass: "prev-fire-time-column",
                        mData: null,
                        mRender: renderPrevFireTimeColumn,
                        bSortable: true,
                        bVisible: false,
                        aTargets: [14],
                        sName: "prevFireTimeColumn",
                        type: 'html'
                    },
                    {
                        bSearchable: false,
                        sClass: "next-fire-time-column",
                        mData: null,
                        mRender: renderNextFireTimeColumn,
                        bSortable: true,
                        bVisible: false,
                        aTargets: [15],
                        sName: "nextFireTimeColumn",
                        type: 'html'
                    },
                    {
                        bSearchable: false,
                        sClass: "fire-count-column",
                        mData: null,
                        mRender: renderFireCountColumn,
                        bSortable: true,
                        bVisible: false,
                        aTargets: [16],
                        sName: "fireCountColumn",
                        type: 'html'
                    },
                    {
                        bSearchable: false,
                        sClass: "links-column",
                        mData: null,
                        mRender: renderLinksColumn,
                        bSortable: false,
                        bVisible: false,
                        aTargets: [17],
                        sName: "linksColumn",
                        type: 'html'
                    },
                    {
                        sClass: "shared-column",
                        mData: null,
                        mRender: renderSharedColumn,
                        bSortable: false,
                        bVisible: false,
                        aTargets: [18],
                        sName: "sharedWithColumn",
                        type: 'html'
                    }, {
                        bSortable: false,
                        sClass: "due-date-column",
                        mData: null,
                        mRender: renderDueDateColumn,
                        bSearchable: false,
                        bVisible: false,
                        aTargets: [19],
                        sName: "dueDateColumn",
                        type: 'html'
                    }, {
                        bSearchable: false,
                        sClass: "triggers-column",
                        mData: null,
                        mRender: renderTriggersColumn,
                        bSortable: false,
                        aTargets: [20],
                        sName: "triggersColumn",
                        type: 'html'
                    },
                    {
                        bSearchable: false,
                        sClass: "triggers-expressions-column",
                        mData: null,
                        mRender: renderTriggersExpressionsColumn,
                        bSortable: false,
                        bVisible: false,
                        aTargets: [21],
                        sName: "triggersExpressionsColumn",
                        type: 'html'
                    },
                    {
                        bSortable: false,
                        sClass: "actions-column",
                        mData: null,
                        mRender: renderActionColumn,
                        bSearchable: false,
                        aTargets: [22],
                        sName: "actionsColumn",
                        type: 'html'
                    },
                ],
                aaData: [],
                order: [[1, "asc"]]
            });
            table.fnFilter("");
            table.on('page.dt', onPageChange);
            previousPage = table.api().page.info();

            clearAllFilters();

            return table;
        };

        var performanceWarningMessageVisibility = function (visibility) {
            if (visibility == true) {
                AJS.$("#scheduled-issues-table-warning-message").show();
            } else {
                AJS.$("#scheduled-issues-table-warning-message").hide();
            }
        };

        var refreshPerformanceWarningMessage = function () {
            performanceWarningMessageVisibility(false);
            AJS.$.each(scheduledIssueTable.fnGetData(), function (index, issue) {
                if (issue.triggerPerformanceIssues == true && issue.disabled != true) {
                    performanceWarningMessageVisibility(true);
                    return false;
                }
            });
        };

        var showEmptyTableInfo = function () {
            jQueryTS("#tsp-table-with-bulk-buttons").hide();
            jQueryTS("#tsp-empty-table-message").show();
        };

        var hideEmptyTableInfo = function () {
            jQueryTS("#tsp-table-with-bulk-buttons").show();
            jQueryTS("#tsp-empty-table-message").hide();
        };

        /* INSTANCE INITIALIZATION */
        scheduledIssueTable = initDataTable();

        scheduledIssueTable.fnSetColumnVis(1, false);
        scheduledIssueTable.fnAdjustColumnSizing();
        scheduledIssueTable.width("100%");

        scheduledIssueTable.fnSetColumnVis(1, true);
        scheduledIssueTable.fnAdjustColumnSizing();
        scheduledIssueTable.width("100%");

        scheduledIssueTable.on('draw.dt', updateVisibleRowsCounter);

        AJS.$('.errors-tooltip-enabled').tooltip({
            live: true,
            html: true,
            title: function () {

                var scheduledIssueId = jQueryTS(this).attr("id").replace("errors-tooltip-", "");
                var validationResult = {};

                var ajaxRequest = jQueryTS.ajax({
                    "type": "GET",
                    "url": AJS.contextPath() + "/rest/thescheduler/1.0/validations/scheduled-issue/" + scheduledIssueId,
                    "dataType": "json",
                    "async": false
                });

                ajaxRequest.done(function (data) {
                    validationResult = data;
                });

                return TheScheduler.Templates.ScheduledIssueTable.errorsTooltip({
                    "validationResult": validationResult
                });

            }
        });

        AJS.$('.tooltip-performance').tooltip({
            live: true,
            html: true,
            title: function () {
                return TheScheduler.Templates.ScheduledIssueTable.warningsTooltip({
                    "message": AJS.I18n.getText('pl.com.tt.thescheduler.tabpanel.performance-warning.selected')
                });

            }
        });

        function onExpandTriggersClick(event) {

            var expandLiElement = jQueryTS(this).parent();
            var ulElement = expandLiElement.parent();
            jQueryTS("li", ulElement).show();

            expandLiElement.hide();
            jQueryTS("li.collapse-triggers", ulElement).show();

            return false;
        }

        jQueryTS(scheduledIssueTableId).off("click", "li.expand-triggers a", onExpandTriggersClick);
        jQueryTS(scheduledIssueTableId).on("click", "li.expand-triggers a", onExpandTriggersClick);

        function onCollapseTriggersClick(event) {

            var collapseLiElement = jQueryTS(this).parent();
            var ulElement = collapseLiElement.parent();
            jQueryTS("li", ulElement).each(function (index) {
                if (index >= 2) {
                    jQueryTS(this).hide();
                }
            });

            collapseLiElement.hide();
            jQueryTS("li.expand-triggers", ulElement).show();

            return false;
        }

        jQueryTS(scheduledIssueTableId).off("click", "li.collapse-triggers a", onCollapseTriggersClick);
        jQueryTS(scheduledIssueTableId).on("click", "li.collapse-triggers a", onCollapseTriggersClick);

        if (userHasNotClosedMessage()) {
            showMessageIfThereAreHiddenScheduledIssues();
        }

        function userHasNotClosedMessage() {
            return jQueryTS.cookie(hiddenScheduledIssuesMessageClosedCookieName) === undefined;
        }

        function showMessageIfThereAreHiddenScheduledIssues() {
            if (projectId.value) {
                jQueryTS.ajax({
                    "type": "GET",
                    "url": AJS.contextPath() + "/rest/thescheduler/1.0/project/" + projectId.value + "/properties",
                    "dataType": "json",
                    "async": true
                }).done(function (data) {
                    if (data.hideInaccessibleSI === true) {
                        showMessageAboutHiddenScheduledIssues();
                    }
                });
            }
        }

        function showMessageAboutHiddenScheduledIssues() {
            var messageId = 'scheduled-issues-table-messages-hidden-scheduled-issues-info';

            AJS.messages.info("#scheduled-issues-table-messages", {
                id: messageId,
                body: '<div>' +
                    AJS.I18n.getText('pl.com.tt.thescheduler.tabpanel.hide-si-info') +
                    '<ul><li>' + AJS.I18n.getText('pl.com.tt.thescheduler.tabpanel.hide-si-info.manage-visibility-settings') + '</li>' +
                    '<li>' + AJS.I18n.getText('pl.com.tt.thescheduler.tabpanel.hide-si-info.grant-access') + '</li>' +
                    '<li>' + AJS.I18n.getText('pl.com.tt.thescheduler.tabpanel.hide-si-info.admin-all-access') + '</li></ul>' + '</div>',
                isCloseable: true
            });

            // listening on 'aui-message-close' event doesn't work because listener gets no information
            // which message was closed so it's not guaranteed that it was actually this message
            AJS.$('#' + messageId).click(function () {
                jQueryTS.cookie(hiddenScheduledIssuesMessageClosedCookieName, 'true', {expires: expiresInYear});
            });
        }

        AJS.$("#scheduled-issues-table-messages").append(TheScheduler.Templates.ScheduledIssueTable.performanceWarningMessage({}));
        AJS.$("#scheduled-issues-table_paginate").appendTo(AJS.$("#tsp-pagination"));

        function licenseCheckRequest() {
            return jQueryTS.ajax({
                "type": "GET",
                "url": AJS.contextPath() + '/rest/thescheduler/1.0/validations/license',
                "async": true,
            });
        }

        /* RETURNING ONLY DESIRABLE MEMBERS AND METHODS */

        return {
            "getScheduledIssueTableId": getScheduledIssueTableId,
            "getDataTable": getDataTable,
            "selectAll": selectAll,
            "unselectAll": unselectAll,
            "getSelectedRows": getSelectedRows,
            "getSelectedScheduledIssues": getSelectedScheduledIssues,
            "addScheduledIssue": addScheduledIssue,
            "deleteScheduledIssue": deleteScheduledIssue,
            "getScheduledIssues": getScheduledIssues,
            "getVisibleColumnIds": getVisibleColumnIds,
            "getVisibleColumnsNames": getVisibleColumnsNames,
            "getColumnVisibilityTableAfterRefreshClick": getColumnVisibilityTableAfterRefreshClick,
            "setColumnVisible": setColumnVisible,
            "setColumnVisibleWithoutRedraw": setColumnVisibleWithoutRedraw,
            "setColumnVisibleByNameWithoutRedraw": setColumnVisibleByNameWithoutRedraw,
            "refresh": projectId ? refreshProjectSITable : refreshGlobalSITable,
            "redraw": redraw,
            "displayLengthChange": displayLengthChange,
            "clearAndRedraw": clearAndRedraw,
            "showEmptyTableInfo": showEmptyTableInfo,
            "hideEmptyTableInfo": hideEmptyTableInfo,
            "updateSelectedRowsCounters": updateSelectedRowsCounters
        };
    };
})();
