TheScheduler.namespace("TheScheduler.View.ShowCreatedIssuesDialog");

TheScheduler.View.ShowCreatedIssuesDialog = (function () {

    /* DECLARING DEPENDENCIES */

    /* STATIC PRIVATE MEMBERS */
    var ColumnIndex = {
        EXECUTION_TIME: 0,
        SCHEDULED_JOB: 1,
        PROJECT: 2,
        EXECUTION_TYPE: 3,
        EXECUTION_RESULT: 4,
        ISSUE_KEY_AND_STATUS: 5
    }
    /* STATIC PRIVATE METHODS */

    /* STATIC INITIALIZATION */

    /* RETURNING AND OVERRIDING CONSTRUCTOR */
    return function (_selectedScheduledIssuesIdsArray, _selectedScheduledIssuesNamesArray, _projectArray, _projectName) {

        var ShowCreatedIssuesDialog = TheScheduler.namespace("TheScheduler.View.ShowCreatedIssuesDialog");
        var CurrentIsoDate = TheScheduler.namespace("TheScheduler.Utils.DateUtils");
        var Common = TheScheduler.namespace("TheScheduler.Utils.Common")

        /* IN CASE SOMEONE FORGOT USING *NEW* KEYWORD */
        if (!(this instanceof ShowCreatedIssuesDialog)) {
            return new ShowCreatedIssuesDialog(_selectedScheduledIssuesIdsArray, _selectedScheduledIssuesNamesArray, _projectArray, _projectName);
        }

        /* PRIVATE MEMBERS */
        var projectId = _projectArray[0];
        var projectArray = _projectArray;
        var projectName = _projectName;
        var selectedScheduledIssuesIdsArray = _selectedScheduledIssuesIdsArray;
        var selectedScheduledIssuesNamesArray = _selectedScheduledIssuesNamesArray;
        var dialogId = "#tsp-si-show-created-issues-dialog";
        var triggerTableId = "#created-issues-table";
        var $dialog = null;
        var table = null;
        var finishTime = null;
        var limit = 10;
        var tableType = "dialog";

        /* PRIVATE METHODS */
        var getDialogId = function () {
            return dialogId;
        };

        var show = function () {
            $dialog.show();
        };

        var hide = function () {
            table.fnDestroy();
            setTimeout(function() {
                AJS.$(".scheduler-tooltip-enabled").tooltip("hide");//Fix for non-hiding tooltips on dialog triggering button
            }, 1000);
        };

        var initDialog = function () {
          var selectedScheduledIssuesJoinedArray = '';
          var selectedScheduledIssuesArrayLength = 0;
          if (selectedScheduledIssuesNamesArray) {
            selectedScheduledIssuesJoinedArray = selectedScheduledIssuesNamesArray.join(', ');
            selectedScheduledIssuesArrayLength = selectedScheduledIssuesNamesArray.length;
          }
            var ShowCreatedIssuesPanelContent = TheScheduler.Templates.Dialogs.showCreatedIssuesPanel({
                "scheduledIssueName": selectedScheduledIssuesJoinedArray,
                "selectedSILength": selectedScheduledIssuesArrayLength,
                "projectName": projectName,
                "maxDate": CurrentIsoDate.getCurrentIsoDate
            });

            AJS.$("body").append(ShowCreatedIssuesPanelContent);
            var $dialog = AJS.dialog2(dialogId);

            AJS.$("#tsp-si-show-created-issues-close-button").click(function (e) {
                $dialog.hide();
            });

            AJS.$('#created-issues-table-blanket').show();

            AJS.dialog2("#tsp-si-show-created-issues-dialog").on("hide", function (e) {
                hide();
            });

            AJS.$("#created-issues-filter-input").bind("input", AJS.$.debounce(250, function () {
                filter(AJS.$(this).val());
            }));

            AJS.$('#created-issues-limit-amount').change(onLimitChange);

            AJS.$(".created-issues-dialog-tooltip-enabled").tooltip();

            getOptionsForSelect();

            AJS.$("#created-issues-limit-by-amount").click(function () {
                AJS.$('#created-issues-limit-date-picker').hide();
                AJS.$('#total-amount-created-issues').hide();
                AJS.$("#created-issues-limit-amount-picker").show();
                refresh();
            });
          AJS.$('#created-issues-limit-date-picker').hide();

          AJS.$('#created-issues-limit-by-date')
            .click(function () {
              AJS.$('#created-issues-limit-amount-picker')
                .hide();
              AJS.$('#created-issues-limit-date-picker')
                .show();
              AJS.$('#total-amount-created-issues')
                .show();

              const tableData = table.api()
                .data();
              const entriesLength = tableData.length;

              var dateValue = new Date();
              if (entriesLength) {
                const lastEntryTimestamp = tableData.toArray()
                  .reverse()
                  .map(function (entry) {
                    return entry.createdTimestamp;
                  })
                  .find(function (executionTimestamp) {
                      if (executionTimestamp) {
                        return true;
                      }
                      return false;
                    }
                  );
                if (lastEntryTimestamp) {
                  dateValue = new Date(lastEntryTimestamp);
                }
              }

              AJS.$('#created-issues-limit-date')
                .val(dateValue.toISOString()
                  .slice(0, 10));
              refreshWithLimitDate();
            });

            AJS.$("#created-issues-limit-date").datePicker({
                overrideBrowserDefault: true,
                firstDay: -1
            });

            AJS.$("#created-issues-limit-date").change(refreshWithLimitDate);

            var selectedSINamesString = '';
            if (selectedScheduledIssuesNamesArray !== null && selectedScheduledIssuesNamesArray.length > 0) {
                selectedSINamesString = Array.from(new Set(selectedScheduledIssuesNamesArray)).map(function(name) {
                  const escapedString = name.replace(/["']/g, '\\$&');
                  return '\"' + encodeURIComponent(escapedString) + '\"';
                });
            }

            AJS.$('#tsp-jql-redirect-button').click(function () {
                if (selectedScheduledIssuesNamesArray) {
                    window.location.href = AJS.contextPath() + '/issues/?jql=issue%20in%20schedulerGetGeneratedIssuesByName(' + selectedSINamesString + ')%20AND%20project%20in%20(' + Array.from(new Set(projectArray)) + ')';
                } else {
                    window.location.href = AJS.contextPath() + '/issues/?jql=issue%20in%20schedulerGetAllGeneratedIssues(' + projectId + ')';
                }
            });

            return $dialog;
        };

        var filter = function (text) {
            table.fnFilter(text);
        }

        function getCreatedIssuesUrl() {
            var url = null;
            if(selectedScheduledIssuesIdsArray) {
                var queryParams = "?scheduledIssueId=" + selectedScheduledIssuesIdsArray.join("&scheduledIssueId=");
                url = '/rest/thescheduler/1.0/scheduled-issue/created-issues-selected' + queryParams + '&limit=' + limit;
            }
            else {
                if (projectId === null) {
                    url = '/rest/thescheduler/1.0/project/created-issues?limit=' + limit;
                } else {
                    url = '/rest/thescheduler/1.0/project/created-issues?projectId=' + projectId + '&limit=' + limit;
                }
            }
            return url;
        }

        function getCreatedIssuesSinceStartDateUrl() {
            var url = null;
            if(selectedScheduledIssuesIdsArray) {
                var queryParams = "?scheduledIssueId=" + selectedScheduledIssuesIdsArray.join("&scheduledIssueId=");
                url = '/rest/thescheduler/1.0/scheduled-issue/created-issues-selected/limit' + queryParams + '&date=' +  AJS.$("#created-issues-limit-date").val();
            }else {
                if (projectId === null) {
                    url = '/rest/thescheduler/1.0/project/created-issues/limit?date=' +  AJS.$("#created-issues-limit-date").val();
                } else {
                    url = '/rest/thescheduler/1.0/project/created-issues/limit?projectId=' + projectId + '&date=' +  AJS.$("#created-issues-limit-date").val();
                }
            }

            return url;
        }

        function onLimitChange() {
            limit = parseInt(AJS.$('#created-issues-limit-amount').val(), 10);
            refresh();
        }

        var refreshWithLimitDate = function (refreshCompletedCallback) {
            AJS.$('#created-issues-limit-date').attr('disabled', '');

            AJS.$('#performance-warning-message').remove();
            var ajaxRequest = jQueryTS.ajax({
                'type': 'GET',
                'url': AJS.contextPath() + getCreatedIssuesSinceStartDateUrl(),
                'dataType': 'json',
                'async': true
            });

            ajaxRequest.done(function (data) {
                table.fnClearTable(true);
                if (AJS.$.isArray(data.entries) && data.entries.length > 0) {
                    table.fnAddData(data.entries, true);
                }
                if (data.limitReached) {
                    const messages = [AJS.I18n.getText("pl.com.tt.thescheduler.errors.issues.limit", data.entries.length),
                        AJS.I18n.getText('pl.com.tt.thescheduler.errors.created-issues.over-limit')];
                    Common.showLimitReachedFlag(messages);
                }
                AJS.$("#total-amount-created-issues").text(AJS.I18n.getText('pl.com.tt.thescheduler.dialogs.upcoming-executions.total') + ": " + (data.entries.length));
            });

            ajaxRequest.always(function () {
                if (typeof refreshCompletedCallback === 'function') {
                    refreshCompletedCallback();
                }
                AJS.$('#created-issues-limit-date').removeAttr('disabled');
            });
        };

        var refresh = function (refreshCompletedCallback) {
            AJS.$('#created-issues-loading').show();
            AJS.$('#created-issues-limit-amount').attr('disabled', '');

            AJS.$('#performance-warning-message').remove();
            var ajaxRequest = jQueryTS.ajax({
                'type': 'GET',
                'url': AJS.contextPath() + getCreatedIssuesUrl(),
                'dataType': 'json',
                'async': true
            });

            ajaxRequest.done(function (data) {
                table.fnClearTable(true);
                if (AJS.$.isArray(data.entries) && data.entries.length > 0) {
                    table.fnAddData(data.entries, true);
                }
            });

            ajaxRequest.always(function () {
                if (typeof refreshCompletedCallback === 'function') {
                    refreshCompletedCallback();
                }
                AJS.$('#created-issues-loading').hide();
                AJS.$('#created-issues-table-blanket').hide();
                AJS.$('#created-issues-limit-amount').removeAttr('disabled');
            });
        };

        var getOptionsForSelect = function(appendCompletedCallback) {
            var url = AJS.contextPath() + "/rest/thescheduler/1.0/global/properties";
            var defaultValues = [10, 20, 50, 100, 200, 500];

            var ajaxRequest = jQueryTS.ajax({
                "type": "GET",
                "url": url,
                "dataType": "json",
                "async": true
            });

            ajaxRequest.done(function (data) {
                for(i=0; i<=defaultValues.length-1; i++){
                    if(defaultValues[i] <= data.maxAmountCreatedIssues){
                        if(i === 0){
                            AJS.$('#created-issues-limit-amount').append('<option value=\"' + defaultValues[i] + '\" selected=\"selected\">' + defaultValues[i] + '</option>');
                            continue;
                        }
                        if(defaultValues[i] === data.maxAmountCreatedIssues){
                            AJS.$('#created-issues-limit-amount').append('<option value=\"' + defaultValues[i] + '\">' + defaultValues[i] + '</option>');
                            break;
                        }
                        AJS.$('#created-issues-limit-amount').append('<option value=\"' + defaultValues[i] + '\">' + defaultValues[i] + '</option>');
                    }

                    if(i > 0 && i < defaultValues.length-1 && defaultValues[i+1] > data.maxAmountCreatedIssues){
                        AJS.$('#created-issues-limit-amount').append('<option value=\"' +  data.maxAmountCreatedIssues + '\">' +  data.maxAmountCreatedIssues + '</option>');
                        break;
                    }

                    if(i === defaultValues.length-1 && defaultValues[i] < data.maxAmountCreatedIssues){
                        AJS.$('#created-issues-limit-amount').append('<option value=\"' +  data.maxAmountCreatedIssues + '\">' +  data.maxAmountCreatedIssues + '</option>');
                        break;
                    }
                }
            });
        };

        function renderProjectColumn(data, type, row) {
            return TheScheduler.Templates.Dialogs.projectColumn({
                createdIssue: row,
                contextPath: AJS.contextPath()
            });
        }

        function renderIssueKeyAndStatusColumn(data, type, row) {
            return TheScheduler.Templates.Dialogs.issueKeyAndStatusColumn({
                "createdIssue": row,
                "contextPath": AJS.contextPath(),
                "issueSortableName": row.issueKey,
                "tableType": tableType,
            });
        }

        function renderScheduledJobColumn(data, type, row) {
            return TheScheduler.Templates.Dialogs.scheduledJobColumn({
                "createdIssue": row,
                "contextPath": AJS.contextPath()
            });
        }

        function renderExecutionTypeColumn(data, type, row) {
            return TheScheduler.Templates.Dialogs.executionTypeColumn({
                "createdIssue": row,
                "contextPath": AJS.contextPath()
            });
        }

        function renderExecutionResultColumn(data, type, row) {
            return TheScheduler.Templates.Dialogs.executionResultColumn({
                "createdIssue": row,
                dialogType: 'CREATED',
            });
        }

        function createdTimeStringColumn(data, type, row) {
            return TheScheduler.Templates.Dialogs.creationTimeColumn({
                "createdIssue": row
            });
        }

        function pad(num, size) {
            var s = "0000000000" + num;
            return s.substr(s.length - size);
        }

        function drawCallback(oSettings) {
            if (oSettings.fnRecordsDisplay() == 0) {
                AJS.$("#created-issues-table_paginate").hide();
            } else {
                AJS.$("#created-issues-table_paginate").show();
            }
        }

        var initDataTable = function () {
            return jQueryTS(triggerTableId).dataTable({
                bSort: true,
                bSortClasses: false,
                bFilter: true,
                bInfo: false,
                bLengthChange: false,
                bPaginate: true,
                sPaginationType: "full_numbers",
                bAutoWidth: true,
                sDom: "lrtip",
                aaSorting: [[0, "desc"]],
                bDeferRender: true,
                bDestroy: true,
                fnDrawCallback: drawCallback,
                aoColumnDefs: [
                    {sType: "string", sClass: "execution-time-column", mRender: createdTimeStringColumn, aTargets: [ColumnIndex.EXECUTION_TIME]},
                    {sType: "string", sClass: "scheduled-job-column", mRender: renderScheduledJobColumn, aTargets: [ColumnIndex.SCHEDULED_JOB]},
                    {
                        sType: "string",
                        sClass: "project-column",
                        mRender: renderProjectColumn,
                        aTargets: [ColumnIndex.PROJECT],
                        visible: !projectId
                    },
                    {
                        sType: "string",
                        sClass: "execution-type-column",
                        mRender: renderExecutionTypeColumn,
                        aTargets: [ColumnIndex.EXECUTION_TYPE]
                    },
                    {
                        sType: "string",
                        sClass: "execution-result-column",
                        mRender: renderExecutionResultColumn,
                        aTargets: [ColumnIndex.EXECUTION_RESULT]
                    },
                    {sType: "string", sClass: "issue-column", mRender: renderIssueKeyAndStatusColumn, aTargets: [ColumnIndex.ISSUE_KEY_AND_STATUS]}
                ],
                aaData: [],
                oLanguage: {
                    sEmptyTable: AJS.I18n.getText("pl.com.tt.thescheduler.dialogs.no-created-issues"),
                    sZeroRecords: AJS.I18n.getText("pl.com.tt.thescheduler.tabpanel.empty-filter"),
                    oPaginate: {
                        sFirst: AJS.I18n.getText("pl.com.tt.thescheduler.tabpanel.pagination.first"),
                        sLast: AJS.I18n.getText("pl.com.tt.thescheduler.tabpanel.pagination.last"),
                        sNext: AJS.I18n.getText("pl.com.tt.thescheduler.tabpanel.pagination.next"),
                        sPrevious: AJS.I18n.getText("pl.com.tt.thescheduler.tabpanel.pagination.previous")
                    }
                },
            });
        }

        /* INSTANCE INITIALIZATION */
        $dialog = initDialog();
        table = initDataTable();
        refresh();

        /* RETURNING ONLY DESIRABLE MEMBERS AND METHODS */
        return {
            "getDialogId": getDialogId,
            "show": show,
            "hide": hide
        };

    };
})();
