TheScheduler.namespace("TheScheduler.View.TriggerTable");

TheScheduler.View.TriggerTable = (function () {

    /* DECLARING DEPENDENCIES */
    var Trigger = TheScheduler.namespace("TheScheduler.Model.Trigger");

    /* STATIC PRIVATE MEMBERS */

    /* STATIC PRIVATE METHODS */

    /* STATIC INITIALIZATION */

    /* RETURNING AND OVERRIDING CONSTRUCTOR */
    return function (_triggerTableId) {

        var TriggerTable = TheScheduler.namespace("TheScheduler.View.TriggerTable");

        /* IN CASE SOMEONE FORGOT USING *NEW* KEYWORD */
        if (!(this instanceof TriggerTable)) {
            return new TriggerTable(_triggerTableId);
        }

        /* PRIVATE MEMBERS */
        var triggerTableId = _triggerTableId;
        var triggerTable = null;

        /* PRIVATE METHODS */
        var getTriggerTableId = function () {
            return triggerTableId;
        };

        var selectAll = function () {
            var trNodes = triggerTable.fnGetNodes();

            var selectAllCheckbox = jQueryTS("input#select-all", triggerTable);
            selectAllCheckbox.prop("checked", true);

            var allCheckboxes = jQueryTS("input.select-one", trNodes);
            allCheckboxes.each(function () {
                var currentCheckbox = jQueryTS(this);
                currentCheckbox.prop("checked", true);
            });
        };

        var unselectAll = function () {
            var trNodes = triggerTable.fnGetNodes();

            var selectAllCheckbox = jQueryTS("input#select-all", triggerTable);
            selectAllCheckbox.prop("checked", false);

            var allCheckboxes = jQueryTS("input.select-one", trNodes);
            allCheckboxes.each(function () {
                var currentCheckbox = jQueryTS(this);
                currentCheckbox.prop("checked", false);
            });
        };

        var getSelectedRows = function () {

            var trNodes = triggerTable.fnGetNodes();
            var selectedCheckboxes = jQueryTS("input.select-one:checked", trNodes);
            var selectedRows = selectedCheckboxes.closest("tr");

            return selectedRows.toArray();
        };

        var getSelectedTriggers = function () {

            var selectedTriggers = [];

            var selectedRows = getSelectedRows();
            for (var i = 0; i < selectedRows.length; i++) {

                var trigger = triggerTable.fnGetData(selectedRows[i]);
                selectedTriggers.push(trigger);

            }
            return selectedTriggers;
        };

        var addTrigger = function (trigger, onAddTriggerCallback) {
            if (trigger instanceof Trigger) {

                var triggerJSON = trigger.getTriggerJSON();
                var triggerIndexes = triggerTable.fnAddData(triggerJSON, false);
                var triggerIndex = triggerIndexes[0];
                var trRow = triggerTable.fnGetNodes(triggerIndex);

                if (onAddTriggerCallback instanceof Function) {
                    onAddTriggerCallback(triggerIndex, trRow);
                }

                triggerTable.fnDraw();
                return true;
            }
            return false;
        };

        var deleteTrigger = function (trElement, onDeleteTriggerCallback) {

            triggerTable.fnDeleteRow(trElement, null, false);

            if (onDeleteTriggerCallback instanceof Function) {
                onDeleteTriggerCallback(trElement);
            }

            triggerTable.fnDraw();
            return false;
        };

        var getTriggers = function () {
            return triggerTable.fnGetData();
        };

        var initDataTable = function () {

            jQueryTS.fn.dataTableExt.oStdClasses.sSortAsc = "ascending";
            jQueryTS.fn.dataTableExt.oStdClasses.sSortDesc = "descending";
            jQueryTS.fn.dataTableExt.oStdClasses.sSortable = "sortable";

            /* FUNCTION DEFINITIONS RESPONSIBLE FOR RENDERING PARTICULAR PIECE OF TRIGGERS TABLE - ONE FUNCTION FOR EACH TABLE COLUMN */
            function renderSelectColumn(data, type, row) {
                return TheScheduler.Templates.TriggerTable.selectColumn({
                    "trigger": row
                });
            }

            function renderStartDateColumn(data, type, row) {
                return TheScheduler.Templates.TriggerTable.startDateColumn({
                    "trigger": row
                });
            }

            function renderEndDateColumn(data, type, row) {
                return TheScheduler.Templates.TriggerTable.endDateColumn({
                    "trigger": row
                });
            }

            function renderIsSkippingColumn(data, type, row) {
                return TheScheduler.Templates.TriggerTable.isSkippingColumn({
                    "trigger": row
                });
            }

            function renderActionColumn(data, type, row) {
                return TheScheduler.Templates.TriggerTable.actionsColumn({
                    "trigger": row
                });
            }

            function onSelectAllClick(eventObject) {

                var selectAllCheckbox = jQueryTS(this);
                var selectAllChecked = selectAllCheckbox.is(":checked");

                if (selectAllChecked) {
                    selectAll();
                } else {
                    unselectAll();
                }

            }

            jQueryTS(triggerTableId).off("click", "input#select-all", onSelectAllClick);
            jQueryTS(triggerTableId).on("click", "input#select-all", onSelectAllClick);

            function onDeleteTriggerClick(eventObject) {

                eventObject.preventDefault();
                eventObject.stopPropagation();

                var trElement = jQueryTS(this).closest("tr").get(0);
                deleteTrigger(trElement, function (trElement) {

                    var triggers = getTriggers();
                    if (triggers.length == 0) {

                        var triggerTableEmptyDiv = jQueryTS("#trigger-table-empty-div");
                        var triggerTableNotEmptyDiv = jQueryTS("#trigger-table-not-empty-div");

                        triggerTableNotEmptyDiv.hide();
                        triggerTableEmptyDiv.show();
                    }

                });

                var selectAllCheckbox = jQueryTS("input#select-all", triggerTable);
                selectAllCheckbox.prop("checked", false);

                return false;
            }

            jQueryTS(triggerTableId).off("click", "a.show-trigger-fire-times", onShowTriggerFireTimesClick);
            jQueryTS(triggerTableId).on("click", "a.show-trigger-fire-times", onShowTriggerFireTimesClick);
            jQueryTS(triggerTableId).off("click", "a.delete-trigger-link", onDeleteTriggerClick);
            jQueryTS(triggerTableId).on("click", "a.delete-trigger-link", onDeleteTriggerClick);

            function onShowTriggerFireTimesClick(eventObject) {
                var trElement = jQueryTS(this).closest('tr').get(0);
                var trigger = triggerTable.api().row(trElement).data();
                fetchFutureFireTimesForTriggers([trigger], showFutureFireTimesDialog);

                eventObject.preventDefault();
                eventObject.stopPropagation();
                return false;
            }

            function fetchFutureFireTimesForTriggers(triggers, callback) {
                var selectedCalendarId = AJS.$('#calendarId').val();
                var calendarIdQueryParam = '';
                if (selectedCalendarId) {
                    calendarIdQueryParam = '&calendarId=' + selectedCalendarId;
                }

                var ajaxRequest = jQueryTS.ajax({
                    type: 'POST',
                    url: AJS.contextPath() + '/rest/thescheduler/1.0/trigger/fire-times?limit=7' + calendarIdQueryParam,
                    data: jQueryTS.toJSON(triggers),
                    contentType: 'application/json; charset=UTF-8'
                });

                ajaxRequest.done(callback);
            }

            function showFutureFireTimesDialog(fireTimes) {
                var dialogDomElement = TheScheduler.Templates.Dialogs.triggerFutureFireTimesDialog({
                    fireTimes: fireTimes
                });
                AJS.$('body').append(dialogDomElement);
                var dialog = AJS.dialog2('#tsp-trigger-future-fire-times-dialog');

                AJS.$('#tsp-trigger-future-fire-times-dialog-close-button').click(function () {
                    dialog.hide();
                });
                AJS.dialog2('#tsp-trigger-future-fire-times-dialog').on('hide', function () {
                    AJS.$('#tsp-trigger-future-fire-times-dialog').remove();
                });

                AJS.$('#tsp-trigger-future-fire-times-dialog').find('.scheduler-tooltip-enabled').tooltip();

                dialog.show();
            }

            function onDeleteSelectedTriggersClick(eventObject) {
                eventObject.preventDefault();
                eventObject.stopPropagation();

                var selectedRows = getSelectedRows();

                jQueryTS(selectedRows).each(function () {
                    var selectedRow = jQueryTS(this).get(0);
                    deleteTrigger(selectedRow, function (trElement) {

                        var triggers = getTriggers();
                        if (triggers.length == 0) {

                            var triggerTableEmptyDiv = jQueryTS("#trigger-table-empty-div");
                            var triggerTableNotEmptyDiv = jQueryTS("#trigger-table-not-empty-div");

                            triggerTableNotEmptyDiv.hide();
                            triggerTableEmptyDiv.show();
                        }

                    });
                });

                var selectAllCheckbox = jQueryTS("input#select-all", triggerTable);
                selectAllCheckbox.prop("checked", false);

                return false;
            }

            function onShowSelectedFutureFiretimesButtonClick(eventObject) {
                var selectedTriggers = getSelectedTriggers();
                if (selectedTriggers.length === 0) {
                    return showFutureFireTimesDialog([]);
                }

                fetchFutureFireTimesForTriggers(getSelectedTriggers(), showFutureFireTimesDialog);

                eventObject.preventDefault();
                eventObject.stopPropagation();
                return false;
            }

            jQueryTS(triggerTableId).off('click', 'a#triggers-table-show-selected-future-fire-times', onShowSelectedFutureFiretimesButtonClick);
            jQueryTS(triggerTableId).on('click', 'a#triggers-table-show-selected-future-fire-times', onShowSelectedFutureFiretimesButtonClick);
            jQueryTS(triggerTableId).off('click', 'a#delete-selected-triggers-link', onDeleteSelectedTriggersClick);
            jQueryTS(triggerTableId).on('click', 'a#delete-selected-triggers-link', onDeleteSelectedTriggersClick);

            /* HERE WE ARE INITIALIZING DATA TABLE OBJECT FOR TRIGGERS TABLE */
            return jQueryTS(triggerTableId).dataTable({
                bSort: true,
                bFilter: false,
                bInfo: false,
                bLengthChange: false,
                bPaginate: false,
                bAutoWidth: true,
                sDom: "lrtip",
                bDestroy: true,
                aoColumnDefs: [
                    {
                        bSearchable: false,
                        sClass: "select-column",
                        mData: "id",
                        mRender: renderSelectColumn,
                        bSortable: false,
                        aTargets: [0]
                    },
                    {
                        sType: "string",
                        sClass: "trigger-expression-column",
                        mData: "triggerExpression",
                        bSortable: true,
                        aTargets: [1]
                    },
                    {
                        sType: "string",
                        sClass: "trigger-expression-summary-column",
                        mData: "triggerExpressionSummary",
                        bSortable: true,
                        aTargets: [2]
                    },
                    {
                        sType: "string",
                        sClass: "start-date-column",
                        mData: "startDate",
                        mRender: renderStartDateColumn,
                        bSortable: true,
                        aTargets: [3]
                    },
                    {
                        sType: "string",
                        sClass: "end-date-column",
                        mData: "endDate",
                        mRender: renderEndDateColumn,
                        bSortable: true,
                        aTargets: [4]
                    },
                    {
                        sType: "string",
                        sClass: "is-skipping-column",
                        mData: "skipScheduledExecutionAfterManual",
                        mRender: renderIsSkippingColumn,
                        bSortable: false,
                        aTargets: [5]
                    },
                    {
                        sType: "string",
                        bSortable: false,
                        bVisible: false,
                        mData: "recentManualExecutionSj",
                        sClass: "recent-manual-execution-sj-column",
                        aTargets: [6]
                    },
                    {
                        sType: "string",
                        sClass: "time-zone-column",
                        mData: "timeZoneFormatted",
                        bSortable: true,
                        aTargets: [7]
                    },
                    {
                        bSortable: false,
                        sClass: "actions-column",
                        mData: "id",
                        mRender: renderActionColumn,
                        bSearchable: false,
                        aTargets: [8]
                    }
                ],
                aaData: []
            });
        };

        /* INSTANCE INITIALIZATION */
        triggerTable = initDataTable();

        /* RETURNING ONLY DESIRABLE MEMBERS AND METHODS */
        return {
            "getTriggerTableId": getTriggerTableId,
            "selectAll": selectAll,
            "unselectAll": unselectAll,
            "getSelectedRows": getSelectedRows,
            "getSelectedTriggers": getSelectedTriggers,
            "addTrigger": addTrigger,
            "deleteTrigger": deleteTrigger,
            "getTriggers": getTriggers
        };

    };
})();
